export declare type Align = "start" | "center" | "end";
export declare const Align: Align[];
export declare type Anchor = "top_left" | "top_center" | "top_right" | "center_left" | "center" | "center_right" | "bottom_left" | "bottom_center" | "bottom_right";
export declare const Anchor: Anchor[];
export declare type AngleUnits = "deg" | "rad";
export declare const AngleUnits: AngleUnits[];
export declare type BoxOrigin = "corner" | "center";
export declare const BoxOrigin: BoxOrigin[];
export declare type ButtonType = "default" | "primary" | "success" | "warning" | "danger";
export declare const ButtonType: ButtonType[];
export declare type Dimension = "width" | "height";
export declare const Dimension: Dimension[];
export declare type Dimensions = "width" | "height" | "both";
export declare const Dimensions: Dimensions[];
export declare type Direction = "clock" | "anticlock";
export declare const Direction: Direction[];
export declare type Distribution = "uniform" | "normal";
export declare const Distribution: Distribution[];
export declare type FontStyle = "normal" | "italic" | "bold" | "bold italic";
export declare const FontStyle: FontStyle[];
export declare type HatchPatternType = 'blank' | 'dot' | 'ring' | 'horizontal_line' | 'vertical_line' | 'cross' | 'horizontal_dash' | 'vertical_dash' | 'spiral' | 'right_diagonal_line' | 'left_diagonal_line' | 'diagonal_cross' | 'right_diagonal_dash' | 'left_diagonal_dash' | 'horizontal_wave' | 'vertical_wave' | 'criss_cross' | ' ' | '.' | 'o' | '-' | '|' | '+' | '"' | ':' | '@' | '/' | '\\' | 'x' | ',' | '`' | 'v' | '>' | '*';
export declare const HatchPatternType: HatchPatternType[];
export declare type HTTPMethod = "POST" | "GET";
export declare const HTTPMethod: HTTPMethod[];
export declare type HexTileOrientation = "pointytop" | "flattop";
export declare const HexTileOrientation: HexTileOrientation[];
export declare type HoverMode = "mouse" | "hline" | "vline";
export declare const HoverMode: HoverMode[];
export declare type LatLon = "lat" | "lon";
export declare const LatLon: LatLon[];
export declare type LegendClickPolicy = "none" | "hide" | "mute";
export declare const LegendClickPolicy: LegendClickPolicy[];
export declare type LegendLocation = Anchor;
export declare const LegendLocation: LegendLocation[];
export declare type LineCap = "butt" | "round" | "square";
export declare const LineCap: LineCap[];
export declare type LineJoin = "miter" | "round" | "bevel";
export declare const LineJoin: LineJoin[];
export declare type LinePolicy = "prev" | "next" | "nearest" | "interp" | "none";
export declare const LinePolicy: LinePolicy[];
export declare type Location = "above" | "below" | "left" | "right";
export declare const Location: Location[];
export declare type Logo = "normal" | "grey";
export declare const Logo: Logo[];
export declare type MarkerType = "asterisk" | "circle" | "circle_cross" | "circle_x" | "cross" | "dash" | "diamond" | "diamond_cross" | "hex" | "inverted_triangle" | "square" | "square_cross" | "square_x" | "triangle" | "x";
export declare const MarkerType: MarkerType[];
export declare type Orientation = "vertical" | "horizontal";
export declare const Orientation: Orientation[];
export declare type OutputBackend = "canvas" | "svg" | "webgl";
export declare const OutputBackend: OutputBackend[];
export declare type PaddingUnits = "percent" | "absolute";
export declare const PaddingUnits: PaddingUnits[];
export declare type Place = Side | "center";
export declare const Place: Place[];
export declare type PointPolicy = "snap_to_data" | "follow_mouse" | "none";
export declare const PointPolicy: PointPolicy[];
export declare type RadiusDimension = "x" | "y" | "max" | "min";
export declare const RadiusDimension: RadiusDimension[];
export declare type RenderLevel = "image" | "underlay" | "glyph" | "annotation" | "overlay";
export declare const RenderLevel: RenderLevel[];
export declare type RenderMode = "canvas" | "css";
export declare const RenderMode: RenderMode[];
export declare type ResetPolicy = "standard" | "event_only";
export declare const ResetPolicy: ResetPolicy[];
export declare type RoundingFunction = "round" | "nearest" | "floor" | "rounddown" | "ceil" | "roundup";
export declare const RoundingFunction: RoundingFunction[];
export declare type Side = "above" | "below" | "left" | "right";
export declare const Side: Side[];
export declare type SizingMode = "stretch_width" | "stretch_height" | "stretch_both" | "scale_width" | "scale_height" | "scale_both" | "fixed";
export declare const SizingMode: SizingMode[];
export declare type SliderCallbackPolicy = "continuous" | "throttle" | "mouseup";
export declare const SliderCallbackPolicy: SliderCallbackPolicy[];
export declare type Sort = "ascending" | "descending";
export declare const Sort: Sort[];
export declare type SpatialUnits = "screen" | "data";
export declare const SpatialUnits: SpatialUnits[];
export declare type StartEnd = "start" | "end";
export declare const StartEnd: StartEnd[];
export declare type StepMode = "after" | "before" | "center";
export declare const StepMode: StepMode[];
export declare type TapBehavior = "select" | "inspect";
export declare const TapBehavior: TapBehavior[];
export declare type TextAlign = "left" | "right" | "center";
export declare const TextAlign: TextAlign[];
export declare type TextBaseline = "top" | "middle" | "bottom" | "alphabetic" | "hanging" | "ideographic";
export declare const TextBaseline: TextBaseline[];
export declare type TextureRepetition = "repeat" | "repeat_x" | "repeat_y" | "no_repeat";
export declare const TextureRepetition: TextureRepetition[];
export declare type TickLabelOrientation = "vertical" | "horizontal" | "parallel" | "normal";
export declare const TickLabelOrientation: TickLabelOrientation[];
export declare type TooltipAttachment = "horizontal" | "vertical" | "left" | "right" | "above" | "below";
export declare const TooltipAttachment: TooltipAttachment[];
export declare type UpdateMode = "replace" | "append";
export declare const UpdateMode: UpdateMode[];
export declare type VerticalAlign = "top" | "middle" | "bottom";
export declare const VerticalAlign: VerticalAlign[];
