"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Widgets = require("./index");
exports.Widgets = Widgets;
var base_1 = require("../../base");
base_1.register_models(Widgets);
